@@ -1,6 +1,6 @@
 /* File Transfer Protocol support.
-   Copyright (C) 1996-2011, 2014-2015, 2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 1996-2011, 2014-2015, 2018-2020 Free Software
+   Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -400,7 +400,7 @@ getftp (struct url *u, struct url *original_url,
       if (!ssl_init ())
         {
           scheme_disable (SCHEME_FTPS);
-          logprintf (LOG_NOTQUIET, _("Could not initialize SSL. It will be disabled."));
+          logprintf (LOG_NOTQUIET, _("Could not initialize SSL. It will be disabled.\n"));
           err = SSLINITFAILED;
           return err;
         }
@@ -756,12 +756,12 @@ Error in server response, closing control connection.\n"));
       else
         {
           const char *targ = NULL;
+          char *target = u->dir;
+          char targetbuf[1024];
           int cwd_count;
           int cwd_end;
           int cwd_start;
 
-          char *target = u->dir;
-
           DEBUGP (("changing working directory\n"));
 
           /* Change working directory.  To change to a non-absolute
@@ -799,13 +799,20 @@ Error in server response, closing control connection.\n"));
               && (con->rs != ST_OS400)
               && (con->rs != ST_VMS))
             {
-              int idlen = strlen (con->id);
               char *ntarget, *p;
+              size_t idlen = strlen (con->id);
+              size_t len;
 
               /* Strip trailing slash(es) from con->id. */
               while (idlen > 0 && con->id[idlen - 1] == '/')
                 --idlen;
-              p = ntarget = (char *)alloca (idlen + 1 + strlen (u->dir) + 1);
+
+              len = idlen + 1 + strlen (target);
+              if (len < sizeof (targetbuf))
+                p = ntarget = targetbuf;
+              else
+                p = ntarget = xmalloc (len + 1);
+
               memcpy (p, con->id, idlen);
               p += idlen;
               *p++ = '/';
@@ -985,6 +992,9 @@ Error in server response, closing control connection.\n"));
 
           /* 2004-09-20 SMS. */
 
+          if (target != targetbuf)
+            xfree (target);
+
         } /* else */
     }
   else /* do not CWD */
@@ -1830,7 +1840,7 @@ ftp_loop_internal (struct url *u, struct url *original_url, struct fileinfo *f,
   /* Declare WARC variables. */
   bool warc_enabled = (opt.warc_filename != NULL);
   FILE *warc_tmp = NULL;
-  ip_address *warc_ip = NULL;
+  ip_address warc_ip_buf, *warc_ip = NULL;
   wgint last_expected_bytes = 0;
 
   /* Get the target, and set the name for the message accordingly. */
@@ -1912,7 +1922,7 @@ ftp_loop_internal (struct url *u, struct url *original_url, struct fileinfo *f,
 
           if (!con->proxy && con->csock != -1)
             {
-              warc_ip = (ip_address *) alloca (sizeof (ip_address));
+              warc_ip = &warc_ip_buf;
               socket_ip_address (con->csock, warc_ip, ENDPOINT_PEER);
             }
         }
@@ -1974,6 +1984,7 @@ ftp_loop_internal (struct url *u, struct url *original_url, struct fileinfo *f,
         case HOSTERR: case CONIMPOSSIBLE: case FWRITEERR: case FOPENERR:
         case FTPNSFOD: case FTPLOGINC: case FTPNOPASV: case FTPNOAUTH: case FTPNOPBSZ: case FTPNOPROT:
         case UNLINKERR: case WARC_TMP_FWRITEERR: case CONSSLERR: case CONTNOTSUPPORTED:
+        case VERIFCERTERR:
 #ifdef HAVE_SSL
           if (err == FTPNOAUTH)
             logputs (LOG_NOTQUIET, "Server does not support AUTH TLS.\n");
@@ -2184,7 +2195,7 @@ ftp_get_listing (struct url *u, struct url *original_url, ccon *con,
 static uerr_t ftp_retrieve_dirs (struct url *, struct url *,
                                  struct fileinfo *, ccon *);
 static uerr_t ftp_retrieve_glob (struct url *, struct url *, ccon *, int);
-static struct fileinfo *delelement (struct fileinfo *, struct fileinfo **);
+static struct fileinfo *delelement (struct fileinfo **, struct fileinfo **);
 
 /* Retrieve a list of files given in struct fileinfo linked list.  If
    a file is a symbolic link, do not retrieve it, but rather try to
@@ -2325,10 +2336,22 @@ The sizes do not match (local %s) -- retrieving.\n\n"),
                       size_t len = strlen (f->linkto) + 1;
                       if (S_ISLNK (st.st_mode))
                         {
-                          char *link_target = (char *)alloca (len);
-                          size_t n = readlink (con->target, link_target, len);
-                          if ((n == len - 1)
-                              && (memcmp (link_target, f->linkto, n) == 0))
+                          char buf[1024], *link_target;
+                          size_t n;
+                          bool res;
+
+                          if (len < sizeof (buf))
+                            link_target = buf;
+                          else
+                            link_target = xmalloc (len);
+
+                          n = readlink (con->target, link_target, len);
+                          res = (n == len - 1) && (memcmp (link_target, f->linkto, n) == 0);
+
+                          if (link_target != buf)
+                            xfree (link_target);
+
+                          if (res)
                             {
                               logprintf (LOG_VERBOSE, _("\
 Already have correct symlink %s -> %s\n\n"),
@@ -2460,8 +2483,9 @@ static uerr_t
 ftp_retrieve_dirs (struct url *u, struct url *original_url,
                    struct fileinfo *f, ccon *con)
 {
-  char *container = NULL;
-  int container_size = 0;
+  char buf[1024];
+  char *container = buf;
+  int container_size = sizeof (buf);
 
   for (; f; f = f->next)
     {
@@ -2478,7 +2502,14 @@ ftp_retrieve_dirs (struct url *u, struct url *original_url,
          item on the bottom of the stack.  */
       size = strlen (u->dir) + 1 + strlen (f->name) + 1;
       if (size > container_size)
-        container = (char *)alloca (size);
+        {
+          if (container == buf)
+            container = xmalloc (size);
+          else
+            container = xrealloc (container, size);
+
+          container_size = size;
+        }
       newdir = container;
 
       odir = u->dir;
@@ -2515,6 +2546,9 @@ Not descending to %s as it is excluded/not-included.\n"),
       /* Set the time-stamp?  */
     }
 
+  if (container != buf)
+    xfree (container);
+
   if (opt.quota && total_downloaded_bytes > opt.quota)
     return QUOTEXC;
   else
@@ -2612,7 +2646,7 @@ ftp_retrieve_glob (struct url *u, struct url *original_url,
         {
           logprintf (LOG_VERBOSE, _("Rejecting %s.\n"),
                      quote (f->name));
-          f = delelement (f, &start);
+          f = delelement (&f, &start);
           continue;
         }
 
@@ -2622,15 +2656,34 @@ ftp_retrieve_glob (struct url *u, struct url *original_url,
         {
           logprintf (LOG_VERBOSE, _("Rejecting %s (Invalid Entry).\n"),
                      quote (f->name));
-          f = delelement (f, &start);
+          f = delelement (&f, &start);
           continue;
         }
 
-      if (!accept_url (f->name))
+      if (opt.acceptregex || opt.rejectregex)
         {
-          logprintf (LOG_VERBOSE, _("%s is excluded/not-included through regex.\n"), f->name);
-          f = delelement (f, &start);
-          continue;
+          // accept_url() takes the full URL.
+          char buf[1024];
+          char *url = buf;
+
+          if ((unsigned) snprintf(buf, sizeof(buf), "%s%s%s",
+                                  u->url, f->name, f->type == FT_DIRECTORY ? "/" : "")
+                                  >= sizeof(buf))
+            {
+              url = aprintf("%s%s%s", u->url, f->name, f->type == FT_DIRECTORY ? "/" : "");
+            }
+
+          if (!accept_url (url))
+            {
+              logprintf (LOG_VERBOSE, _ ("%s is excluded/not-included through regex.\n"), url);
+              f = delelement (&f, &start);
+              if (url != buf)
+                xfree(url);
+              continue;
+            }
+
+            if (url != buf)
+              xfree(url);
         }
 
       /* Now weed out the files that do not match our globbing pattern.
@@ -2650,7 +2703,7 @@ ftp_retrieve_glob (struct url *u, struct url *original_url,
                 }
               if (matchres == FNM_NOMATCH)
                 {
-                  f = delelement (f, &start); /* delete the element from the list */
+                  f = delelement (&f, &start); /* delete the element from the list */
                   continue;
                 }
             }
@@ -2658,7 +2711,7 @@ ftp_retrieve_glob (struct url *u, struct url *original_url,
             {
               if (0 != cmp(u->file, f->name))
                 {
-                  f = delelement (f, &start);
+                  f = delelement (&f, &start);
                   continue;
                 }
             }
@@ -2810,14 +2863,15 @@ ftp_loop (struct url *u, struct url *original_url, char **local_file, int *dt,
    address of the next element, or NULL if the list is exhausted.  It
    can modify the start of the list.  */
 static struct fileinfo *
-delelement (struct fileinfo *f, struct fileinfo **start)
+delelement (struct fileinfo **f, struct fileinfo **start)
 {
-  struct fileinfo *prev = f->prev;
-  struct fileinfo *next = f->next;
+  struct fileinfo *prev = (*f)->prev;
+  struct fileinfo *next = (*f)->next;
 
-  xfree (f->name);
-  xfree (f->linkto);
-  xfree (f);
+  xfree ((*f)->name);
+  xfree ((*f)->linkto);
+  xfree (*f);
+  *f = NULL;
 
   if (next)
     next->prev = prev;
